@@ -18,107 +18,107 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-from qt import variantValue, Qt, QPointF, QPoint, QRectF, QMimeData, QSize, toQVariant
-from qt import QGraphicsRectItem, QGraphicsItem, QSizePolicy, QFrame, QFont
-from qt import QGraphicsView, QGraphicsEllipseItem, QLabel
+from qt import Qt, QPointF, QPoint, QRectF, QMimeData, QSize
+from qt import QGraphicsRectItem, QSizePolicy, QFrame, QFont
+from qt import QGraphicsView, QLabel
 from qt import QColor, QPainter, QDrag, QPixmap, QStyleOptionGraphicsItem, QPen, QBrush
-from qt import QFontMetrics, QTransform
+from qt import QFontMetrics, QGraphicsObject
 from qt import QMenu, QCursor
 from qt import QGraphicsSvgItem
-from tileset import Tileset, TileException
+from tileset import Tileset
 from tile import Tile, elements
 from uitile import UITile, UIMeld
-from guiutil import Painter
+from guiutil import Painter, rotateCenter, sceneRotation
 from meld import Meld
-from animation import Animation, MoveImmediate, animate
+from animation import AnimationSpeed, animate, AnimatedMixin
 from message import Message
 
-from util import kprint, stack, uniqueList
-from log import logDebug, logException, m18n, m18nc
-from common import LIGHTSOURCES, Internal, Debug, isAlive, unicode
+from util import stack, uniqueList
+from log import logDebug, logException
+from mi18n import i18n, i18nc
+from common import LIGHTSOURCES, Internal, Debug, isAlive, StrMixin
+from common import ZValues
 from wind import Wind, East
 
-ROUNDWINDCOLOR = QColor(235, 235, 173)
 
-WINDPIXMAPS = {}
-
-
-def rotateCenter(item, angle):
-    """rotates a QGraphicsItem around its center"""
-    center = item.boundingRect().center()
-    centerX, centerY = center.x() * item.scale(), center.y() * item.scale()
-    item.setTransform(QTransform().translate(
-        centerX, centerY).rotate(angle).translate(-centerX, -centerY))
-    return item
-
-
-class PlayerWind(QGraphicsEllipseItem):
+class PlayerWind(AnimatedMixin, QGraphicsObject, StrMixin):
 
     """a round wind tile"""
 
-    def __init__(self, name, tileset, roundsFinished=0, parent=None):
-        """generate new wind tile"""
-        assert isinstance(name, Wind), 'PlayerWind expects Wind, not {}'.format(type(name))
-        if not len(WINDPIXMAPS):
-            WINDPIXMAPS[(East, False)] = None  # avoid recursion
-            self.genWINDPIXMAPS()
-        QGraphicsEllipseItem.__init__(self)
-        if parent:
-            self.setParentItem(parent)
-        self.name = name
-        self.face = QGraphicsSvgItem()
-        self.face.setParentItem(self)
-        self.prevailing = None
-        self.setWind(name, roundsFinished)
-        self.tileset = tileset
-        self.__sizeFace()
+    roundWindColor = QColor(235, 235, 173)
+    whiteColor = QColor('white')
 
-    @staticmethod
-    def genWINDPIXMAPS():
-        """prepare wind tiles"""
-        tileset = Tileset(Internal.Preferences.windTilesetName)
-        for wind in Wind.all4:
-            for prevailing in False, True:
-                pwind = PlayerWind(wind, tileset, prevailing)
-                pMap = QPixmap(40, 40)
-                pMap.fill(Qt.transparent)
-                painter = QPainter(pMap)
-                painter.setRenderHint(QPainter.Antialiasing)
-                painter.scale(0.40, 0.40)
-                pwind.paint(painter, QStyleOptionGraphicsItem())
-                for child in pwind.childItems():
-                    if isinstance(child, QGraphicsSvgItem):
-                        with Painter(painter):
-                            painter.translate(child.mapToParent(0.0, 0.0))
-                            child.paint(painter, QStyleOptionGraphicsItem())
-                WINDPIXMAPS[(wind, prevailing)] = pMap
+    def __init__(self, wind, parent=None):
+        """generate new wind marker"""
+        super(PlayerWind, self).__init__()
+        assert not parent
+        assert isinstance(wind, Wind), 'wind {}  must be a real Wind but is {}'.format(
+            wind, type(wind))
+        self.__wind = wind
+        self.__brush = self.whiteColor
+        self.board = None
 
-    def __sizeFace(self):
-        """size the chinese character depending on the wind tileset"""
-        self.resetTransform()
-        size = self.tileset.faceSize
-        self.setFlag(QGraphicsItem.ItemClipsChildrenToShape)
-        diameter = size.height() * 1.1
-        scaleFactor = 0.9
-        facePos = {'traditional': (10, 10), 'default': (15, 10),
-                   'classic': (19, 1), 'jade': (19, 1)}
-        self.setRect(0, 0, diameter, diameter)
-        self.setScale(scaleFactor)
-        faceX, faceY = facePos[self.tileset.desktopFileName]
-        self.face.setPos(faceX, faceY)
-        self.face.setSharedRenderer(self.tileset.renderer())
+    def name(self):
+        """for identification in animations"""
+        return self.__wind.tile
 
-    def setWind(self, name, roundsFinished):
-        """change the wind"""
-        assert isinstance(name, Wind) and name.svgName, 'name {}  must be a real Wind but is {}'.format(
-            name, type(name))
-        self.name = name
-        if isinstance(roundsFinished, bool):
-            self.prevailing = roundsFinished
+    def moveDict(self):
+        """a dict with attributes for the new position,
+        normally pos, rotation and scale"""
+        sideCenter = self.board.center()
+        boardPos = QPointF(
+            sideCenter.x() * 1.63,
+            sideCenter.y() - self.boundingRect().height() / 2.0)
+        scenePos = self.board.mapToScene(boardPos)
+        return {'pos': scenePos, 'rotation': sceneRotation(self.board)}
+
+    def setDrawingOrder(self):
+        """we want the winds above all others"""
+        if self.activeAnimation.get('pos'):
+            movingZ = ZValues.movingZ
         else:
-            self.prevailing = name == Wind.all4[roundsFinished % 4]
-        self.setBrush(ROUNDWINDCOLOR if self.prevailing else QColor('white'))
-        self.face.setElementId(name.svgName)
+            movingZ = 0
+        self.setZValue(ZValues.markerZ + movingZ)
+
+    @property
+    def wind(self):
+        """our wind"""
+        return self.__wind
+
+    @property
+    def prevailing(self):
+        """is this the prevailing wind?"""
+        return self.__brush is self.roundWindColor
+
+    @prevailing.setter
+    def prevailing(self, value):
+        if isinstance(value, bool):
+            newPrevailing = value
+        else:
+            newPrevailing = self.wind == Wind.all4[value % 4]
+        self.__brush = self.roundWindColor if newPrevailing else self.whiteColor
+
+    def paint(self, painter, dummyOption, dummyWidget=None):
+        """paint the marker"""
+        with Painter(painter):
+            painter.setBrush(self.__brush)
+            size = Internal.scene.windTileset.faceSize.height()
+            ellRect = QRectF(QPoint(), QPoint(size, size))
+            painter.drawEllipse(ellRect)
+            renderer = Internal.scene.windTileset.renderer()
+            painter.translate(12, 12)
+            painter.scale(0.60, 0.60)
+            renderer.render(painter, self.wind.markerSvgName, self.boundingRect())
+
+    def boundingRect(self): # pylint: disable=no-self-use
+        """define the part of the tile we want to see"""
+        size = Internal.scene.windTileset.faceSize.height() * 1.1
+        return QRectF(QPoint(), QPoint(size, size))
+
+    def __str__(self):
+        """for debugging"""
+        return 'WindMarker(%s x/y= %.1f/%1f)' % (
+            self.name(), self.x(), self.y())
 
 
 class WindLabel(QLabel):
@@ -147,24 +147,38 @@ class WindLabel(QLabel):
 
     @property
     def roundsFinished(self):
-        """setting roundsFinished also changes pixmaps if needed"""
+        """setting roundsFinished also changes graphics if needed"""
         return self.__roundsFinished
 
     @roundsFinished.setter
     def roundsFinished(self, roundsFinished):
-        """setting roundsFinished also changes pixmaps if needed"""
+        """setting roundsFinished also changes graphics if needed"""
         if self.__roundsFinished != roundsFinished:
             self.__roundsFinished = roundsFinished
             self._refresh()
 
     def _refresh(self):
-        """update pixmaps"""
-        PlayerWind.genWINDPIXMAPS()
-        self.setPixmap(WINDPIXMAPS[(self.__wind,
-                                    self.__wind == Wind.all4[min(self.__roundsFinished, 3)])])
+        """update graphics"""
+        self.setPixmap(self.genWindPixmap())
 
+    def genWindPixmap(self):
+        """prepare wind tiles"""
+        pwind = PlayerWind(self.__wind)
+        pwind.prevailing = self.__wind == Wind.all4[min(self.__roundsFinished, 3)]
+        pMap = QPixmap(40, 40)
+        pMap.fill(Qt.transparent)
+        painter = QPainter(pMap)
+        painter.setRenderHint(QPainter.Antialiasing)
+        painter.scale(0.40, 0.40)
+        pwind.paint(painter, QStyleOptionGraphicsItem())
+        for child in pwind.childItems():
+            if isinstance(child, QGraphicsSvgItem):
+                with Painter(painter):
+                    painter.translate(child.mapToParent(0.0, 0.0))
+                    child.paint(painter, QStyleOptionGraphicsItem())
+        return pMap
 
-class Board(QGraphicsRectItem):
+class Board(QGraphicsRectItem, StrMixin):
 
     """ a board with any number of positioned tiles"""
     # pylint: disable=too-many-instance-attributes
@@ -199,6 +213,10 @@ class Board(QGraphicsRectItem):
         """default board name, used for debugging messages"""
         return 'board'
 
+    def __str__(self):
+        """for debugging"""
+        return self.name
+
     def setVisible(self, value):
         """also update focusRect if it belongs to this board"""
         if self.scene() and isAlive(self):
@@ -250,7 +268,7 @@ class Board(QGraphicsRectItem):
             self.__prevPos = uiTile.sortKey()
         self._focusTile = uiTile
         if self._focusTile and self._focusTile.tile in Debug.focusable:
-            logDebug(u'%s: new focus uiTile %s from %s' % (
+            logDebug('%s: new focus uiTile %s from %s' % (
                 self.name, self._focusTile.tile if self._focusTile else 'None', stack('')[-1]))
         if self.hasFocus:
             self.scene().focusBoard = self
@@ -294,10 +312,10 @@ class Board(QGraphicsRectItem):
         key = event.key()
         if key in Board.arrows:
             return key
-        charArrows = m18nc(
+        charArrows = i18nc(
             'kajongg:arrow keys hjkl like in konqueror',
             'hjklHJKL')
-        key = unicode(event.text())
+        key = event.text()
         if key and key in charArrows:
             key = Board.arrows[charArrows.index(key) % 4]
         return key
@@ -333,7 +351,7 @@ class Board(QGraphicsRectItem):
     def uiMeldWithTile(self, uiTile):  # pylint: disable=no-self-use
         """returns the UI Meld with uiTile. A Board does not know about melds,
         so default is to return a Meld with only uiTile"""
-        return Meld(uiTile)
+        return UIMeld(uiTile)
 
     def meldVariants(self, uiTile, lowerHalf):  # pylint: disable=no-self-use,unused-argument
         """all possible melds that could be meant by dragging/dropping uiTile"""
@@ -345,10 +363,10 @@ class Board(QGraphicsRectItem):
         variants = self.meldVariants(uiTile, lowerHalf)
         idx = 0
         if len(variants) > 1:
-            menu = QMenu(m18n('Choose from'))
+            menu = QMenu(i18n('Choose from'))
             for idx, variant in enumerate(variants):
                 action = menu.addAction(variant.typeName())
-                action.setData(toQVariant(idx))
+                action.setData(idx)
             if Internal.scene.mainWindow.centralView.dragObject:
                 menuPoint = QCursor.pos()
             else:
@@ -359,7 +377,7 @@ class Board(QGraphicsRectItem):
             action = menu.exec_(menuPoint)
             if not action:
                 return None
-            idx = variantValue(action.data())
+            idx = action.data()
         return variants[idx]
 
     def dragEnterEvent(self, dummyEvent):
@@ -390,7 +408,7 @@ class Board(QGraphicsRectItem):
     def rotatedLightSource(self):
         """the light source we need for the original uiTile before it is rotated"""
         lightSourceIndex = LIGHTSOURCES.index(self.lightSource)
-        lightSourceIndex = (lightSourceIndex + self.sceneRotation() // 90) % 4
+        lightSourceIndex = (lightSourceIndex + sceneRotation(self) // 90) % 4
         return LIGHTSOURCES[lightSourceIndex]
 
     def tileFacePos(self):
@@ -406,20 +424,6 @@ class Board(QGraphicsRectItem):
         """the face rect of a uiTile relative its origin"""
         return QRectF(self.tileFacePos(), self.tileset.faceSize)
 
-    def sceneRotation(self):
-        """the combined rotation of self and all parents"""
-        matrix = self.sceneTransform()
-        matrix = (
-            int(matrix.m11()),
-            int(matrix.m12()),
-            int(matrix.m21()),
-            int(matrix.m22()))
-        rotations = {(0, 0, 0, 0): 0, (1, 0, 0, 1): 0, (
-            0, 1, -1, 0): 90, (-1, 0, 0, -1): 180, (0, -1, 1, 0): 270}
-        if matrix not in rotations:
-            raise Exception('matrix unknown:%s' % str(matrix))
-        return rotations[matrix]
-
     def setPos(self, xWidth=0, xHeight=0, yWidth=0, yHeight=0):
         """sets the position in the parent item expressing the position in tile face units.
         The X position is xWidth*facewidth + xHeight*faceheight, analog for Y"""
@@ -472,7 +476,7 @@ class Board(QGraphicsRectItem):
         else:
             offsets = self.tileset.shadowOffsets(
                 self._lightSource,
-                self.sceneRotation())
+                sceneRotation(self))
         newX = self.__xWidth * width + self.__xHeight * height + offsets[0]
         newY = self.__yWidth * width + self.__yHeight * height + offsets[1]
         QGraphicsRectItem.setPos(self, newX, newY)
@@ -493,10 +497,7 @@ class Board(QGraphicsRectItem):
         """set active lightSource"""
         if self._lightSource != lightSource:
             if lightSource not in LIGHTSOURCES:
-                logException(
-                    TileException(
-                        'lightSource %s illegal' %
-                        lightSource))
+                logException('lightSource %s illegal' % lightSource)
             self._reload(self.tileset, lightSource)
 
     @property
@@ -525,8 +526,7 @@ class Board(QGraphicsRectItem):
             self._lightSource = lightSource
             self.setGeometry()
             for child in self.childItems():
-                if isinstance(child, (Board, PlayerWind)):
-                    child.tileset = tileset
+                if isinstance(child, Board):
                     child.lightSource = lightSource
                     child.showShadows = showShadows
             for uiTile in self.uiTiles:
@@ -571,41 +571,11 @@ class Board(QGraphicsRectItem):
         """the current face size"""
         return self._tileset.faceSize
 
-    def __tilePlace(self, uiTile):
-        """compute all properties for uiTile in this board: pos, scale, rotation
-        and return them in a dict"""
-        assert isinstance(uiTile, UITile)
-        if not uiTile.scene():
-            self.scene().addItem(uiTile)
-        width = self.tileset.faceSize.width()
-        height = self.tileset.faceSize.height()
-        shiftZ = self.shiftZ(uiTile.level)
-        boardPos = QPointF(
-            uiTile.xoffset * width,
-            uiTile.yoffset * height) + shiftZ
-        scenePos = self.mapToScene(boardPos)
-        uiTile.setDrawingOrder()
-        return {'pos': scenePos, 'rotation': self.sceneRotation(), 'scale': self.scale()}
-
     def placeTile(self, uiTile):
-        """places the uiTile in the scene. With direct=False, animate"""
+        """places the uiTile in the scene"""
         assert isinstance(uiTile, UITile)
-        for pName, newValue in self.__tilePlace(uiTile).items():
-            animation = uiTile.queuedAnimation(pName)
-            if animation:
-                curValue = animation.unpackValue(animation.endValue())
-                if curValue != newValue:
-                    # change a queued animation
-                    animation.setEndValue(newValue)
-            else:
-                animation = uiTile.activeAnimation.get(pName, None)
-                if isAlive(animation):
-                    curValue = animation.unpackValue(animation.endValue())
-                else:
-                    curValue = uiTile.getValue(pName)
-                if pName != 'scale' or abs(curValue - newValue) > 0.00001:
-                    if curValue != newValue:
-                        Animation(uiTile, pName, newValue)
+        assert uiTile.board == self
+        uiTile.setupAnimations()
 
     def addUITile(self, uiTile):
         """add uiTile to this board"""
@@ -624,7 +594,7 @@ class CourtBoard(Board):
     penColor = 'green'
 
     def __init__(self, width, height):
-        Board.__init__(self, width, height, Tileset.activeTileset())
+        Board.__init__(self, width, height, Tileset.current())
         self.setAcceptDrops(True)
 
     def maximize(self):
@@ -662,7 +632,6 @@ class SelectorBoard(CourtBoard):
 
     def __init__(self):
         CourtBoard.__init__(self, 9, 5)
-        self.lastReceived = None
         self.allSelectorTiles = []
 
     def checkTiles(self):
@@ -680,7 +649,7 @@ class SelectorBoard(CourtBoard):
 
     def refill(self):
         """move all tiles back into the selector"""
-        with MoveImmediate():
+        with AnimationSpeed():
             for uiTile in self.allSelectorTiles:
                 uiTile.tile = uiTile.tile.exposed
                 self.__placeAvailable(uiTile)
@@ -706,15 +675,18 @@ class SelectorBoard(CourtBoard):
 
     def dropTile(self, uiTile, lowerHalf=False):  # pylint: disable=unused-argument
         """drop uiTile into selector board"""
-        senderHand = uiTile.board
+        uiMeld = uiTile.board.uiMeldWithTile(uiTile)
+        self.dropMeld(uiMeld)
+
+    def dropMeld(self, uiMeld):
+        """drop uiMeld into selector board"""
+        senderHand = uiMeld[0].board
         if senderHand == self:
             return
-        meld = uiTile.board.uiMeldWithTile(uiTile)
-        self.lastReceived = uiTile
-        for myTile in meld:
+        for myTile in uiMeld:
             self.__placeAvailable(myTile)
             myTile.focusable = True
-        senderHand.deselect(meld)
+        senderHand.deselect(uiMeld)
         (senderHand if senderHand.uiTiles else self).hasFocus = True
         self._noPen()
         animate()
@@ -736,11 +708,14 @@ class SelectorBoard(CourtBoard):
     def __placeAvailable(self, uiTile):
         """place the uiTile in the selector at its place"""
         # define coordinates and order for tiles:
-        offsets = {Tile.dragon: (
-            3, 6, Tile.dragons), Tile.flower: (
-                4, 5, Tile.winds), Tile.season: (4, 0, Tile.winds),
-                   Tile.wind: (3, 0, Tile.winds), Tile.bamboo: (1, 0, Tile.numbers), Tile.stone: (2, 0, Tile.numbers),
-                   Tile.character: (0, 0, Tile.numbers)}
+        offsets = {
+            Tile.dragon: (3, 6, Tile.dragons),
+            Tile.flower: (4, 5, Tile.winds),
+            Tile.season: (4, 0, Tile.winds),
+            Tile.wind: (3, 0, Tile.winds),
+            Tile.bamboo: (1, 0, Tile.numbers),
+            Tile.stone: (2, 0, Tile.numbers),
+            Tile.character: (0, 0, Tile.numbers)}
         row, baseColumn, order = offsets[uiTile.tile.lowerGroup]
         column = baseColumn + order.index(uiTile.tile.char)
         uiTile.dark = False
@@ -814,10 +789,11 @@ class FittingView(QGraphicsView):
     def wheelEvent(self, event):  # pylint: disable=no-self-use
         """we do not want scrolling for the scene view.
         Instead scrolling down changes perspective like in kmahjongg"""
-        if event.orientation() == Qt.Vertical and event.delta() < 0:
-            Internal.mainWindow.changeAngle()
-        # otherwise do not call ignore() because we do want
-        # to consume this
+        angleX = event.angleDelta().x()
+        angleY = event.angleDelta().y()
+        if angleX > 15 or angleY > -50:
+            return
+        Internal.mainWindow.changeAngle()
 
     def resizeEvent(self, dummyEvent):
         """scale the scene and its background for new view size"""
@@ -858,7 +834,7 @@ class FittingView(QGraphicsView):
         if tiles:
             if event.modifiers() & Qt.ShiftModifier:
                 for uiTile in tiles:
-                    kprint(
+                    print(
                         '%s: board.level:%s' %
                         (str(uiTile), uiTile.board.level))
             board = tiles[0].board
